 /*
 *
 * TITLE   PNPBIOS.CPP  -  PnP BIOS Runtime Services Library
 *
 * Copyright (c) 1995 by Glenn Jystad, All Rights Reserved.
 *
 ***************************************************************************
 *
 * Filename:         PNPBIOS.CPP
 *
 **************************************************************************/

#define PNPBIOS_CPP

 /*************************************************************************
 *                    Include Files
 **************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "STDDEFS.H"
#include "PNPISA.H"
#include "PNPBIOS.H"

 /*************************************************************************
 *                    Local Defines
 **************************************************************************/

 /*************************************************************************
 *                    Global Variable Declarations
 **************************************************************************/

 /*************************************************************************
 *                    Functions
 **************************************************************************/


 /************************************************************************
 *
 * Function Name: FindPnpBios
 *
 * Description:   Since the $PnP header must be on a paragraph boundary,
 *                we can find it by incrementing a segment value.
 *                If found, we return the segment value.
 *                If not found, we return 0.
 *
 **************************************************************************/
WORD FindPnpBios ()
	{
	PnpHeader far  *fpbyPnpHeader;
	WORD			wSegment;
	//
	//	Note:  Segment == 0 implies we hit the top of 1M address space
	//
	for(wSegment = PNP_SIG_SEARCH_BASE;		// 0xF000
		wSegment != 0x0000;
		wSegment++)
		{
		fpbyPnpHeader = (PnpHeader far *)MK_FP(wSegment, 0);
		if (*(DWORD far *)fpbyPnpHeader == PNP_DW_SIGNATURE)
			{
			BYTE  i;
			BYTE  byCkSum;
			BYTE  byMyLength;

			// $PnP is validated by a 0 sum of bLength bytes.
			byMyLength = fpbyPnpHeader->bLength;
			for(i=0, byCkSum=0; i<byMyLength; i++)
				byCkSum += *(((BYTE far *)fpbyPnpHeader)+i);

			// If checksum passed, we got it.
			if(0==byCkSum) break;
		}// end if
	}// end for

	if ((wSegment) && (fpbyPnpHeader->bLength != 0x21))
		{
		printf ("A PnP Header was found but the length was %d.", fpbyPnpHeader->bLength );
		printf ("A PnPBIOS Spec V1.0A compliant BIOS has a length of 0x21\n");
		}// end if

	return(wSegment);

	}; // FindPnpBios


 /***************************************************************************
 *
 * Function Name: PnpGetHeader
 *
 * Description:   Locate the PnP BIOS $PnP string, validate the checksum
 *                and save the header in a global structure.
 * Parameters:    None.
 * Returns:       0 = Success
 *                1 = Valid header was not found.
 *
 * Notes:		  Version 1.0a of the PnP BIOS Specification specifies that
 *                the size field must be 32 to include the entry points and
 *                data locations.
 *
 *************************************************************************/
int PnpGetHeader ()
	{
	int 	iReturn = 0;
	WORD    wPnpSeg;

	wPnpSeg = FindPnpBios();
	gfpPnpHeader = (PnpHeader far *)MK_FP(wPnpSeg, 0);
	guPnpFunctions.pnpRealModeEntry =
		MK_FP(gfpPnpHeader->wRealCodeSegment,
			 gfpPnpHeader->wRealEntryOffset);
	gwBiosSelector = gfpPnpHeader->wRealDataSegment;
	return (wPnpSeg == 0);

	} // end PnpGetHeader


 /***************************************************************************
 *
 * Function Name: DumpDevNodeHeader
 *
 * Description:   Display the header of a DevNode and move the ptr
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeHeader (BYTE ** ppbDevNode)
	{
	char acBuf[8];

	printf ("\n");
	printf ("  Node #%u\n", (*ppbDevNode)[2]);
	printf ("    Length     : %u\n", *(WORD *)(&(*ppbDevNode)[0]));
	printf ("    ID         : %s\n", EisaId(*(DWORD*)(&(*ppbDevNode)[3]), acBuf));
	printf ("    Type Codes : %02Xh %02Xh %02Xh\n", (*ppbDevNode)[7],(*ppbDevNode)[8],(*ppbDevNode)[9]);
	printf ("    Attribute  : %04Xh\n", *(WORD *)(&(*ppbDevNode)[10]));

	*ppbDevNode += 12;

	} // end DumpDevNodeHeader


 /***************************************************************************
 *
 * Function Name: DumpDevNodeSumHeader
 *
 * Description:   Display the header of a DevNode and moves the ptr
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeSumHeader (BYTE ** ppbDevNode)
	{
	char acBuf[8];

	printf ("%2u - %s :", (*ppbDevNode)[2],
			EisaId(*(DWORD*)(&(*ppbDevNode)[3]), acBuf));

	*ppbDevNode += 12;

	} // end DumpDevNodeSumHeader


 /***************************************************************************
 *
 * Function Name: DumpDevNodeSumAllocated
 *
 * Description:   Display the Allocated resources of a DevNode and move the ptr
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeSumAllocated (BYTE ** ppbDevNode)
	{

	DumpIsaSumResources(ppbDevNode, SUM_SHORT_FORM);

	} // end DumpDevNodeSumAllocated


 /***************************************************************************
 *
 * Function Name: DumpDevNodeAllocated
 *
 * Description:   Display the Allocated resources of a DevNode and move the ptr
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeAllocated (BYTE ** ppbDevNode)
	{
	printf ("  Allocated:\n");
	DumpIsaResources (ppbDevNode);

	} // end DumpDevNodeAllocated


 /***************************************************************************
 *
 * Function Name: DumpDevNodePossibles
 *
 * Description:   Display the Possible Resources of a DevNode and move the ptr
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodePossibles (BYTE ** ppbDevNode)
	{
	printf ("  Possibles:\n");
	DumpIsaResources (ppbDevNode);

	} // end DumpDevNodePossibles


 /***************************************************************************
 *
 * Function Name: DumpDevNodeCompatIds
 *
 * Description:   Display the Compatible Ids of a DevNode
 * Parameters:    BYTE ** ppbDevNode
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeCompatIds (BYTE ** ppbDevNode)
	{
	printf ("  Compatible Ids:\n");
	DumpIsaResources (ppbDevNode);

	} // end DumpDevNodeCompatIds


 /***************************************************************************
 *
 * Function Name: DumpDevNode
 *
 * Description:   Display a DevNode Buffer in parsed format.
 * Parameters:    BYTE * pbDevNode
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNode (BYTE * pbDevNode)
	{
	// Display Header
	DumpDevNodeHeader(&pbDevNode);

	// Display Allocated
	DumpDevNodeAllocated(&pbDevNode);

	// Display Possibles
	DumpDevNodePossibles(&pbDevNode);

	// Display Compatible Ids
	DumpDevNodeCompatIds(&pbDevNode);

	} // end DumpDevNode


 /***************************************************************************
 *
 * Function Name: DumpDevNodeSummaryLine
 *
 * Description:   Display a DevNode Buffer in parsed format.
 * Parameters:    BYTE * pbDevNode
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeSummaryLine (BYTE * pbDevNode)
	{
	// Display Header
	DumpDevNodeSumHeader(&pbDevNode);

	// Display Allocated
	DumpDevNodeSumAllocated(&pbDevNode);

	} // end DumpDevNodeSummaryLine


 /***************************************************************************
 *
 * Function Name: GetDevNode
 *
 * Description:   Get a Device Node's current settings.
 * Parameters:
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
int GetDevNode (WORD * pwDevNode, BYTE * pbBuff)
	{
	int iReturn = 0;

    iReturn = (*guPnpFunctions.fpfnPnpFn1)(PNPBIOS_GETNODE,
				                           (WORD far *)pwDevNode,
				                           pbBuff,
				                           1,
				                           gwBiosSelector);

	return (iReturn);

	} // end GetDevNode


 /***************************************************************************
 *
 * Function Name: DumpDevNodeSummary
 *
 * Description:   Display a DevNode Buffer in parsed format.
 * Parameters:    BYTE * pbDevNode
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void DumpDevNodeSummary ()
	{
	BYTE abDevNode[500];
	BYTE *pbDevNode = abDevNode;
	WORD wDevNode = 0;

	while (GetDevNode(&wDevNode, pbDevNode) == PNPBIOS_ERR_SUCCESS)
		{
		// Display Header
		DumpDevNodeSumHeader(&pbDevNode);

		// Display Allocated
		DumpDevNodeSumAllocated(&pbDevNode);

		pbDevNode = abDevNode;
		}// end while


	} // end DumpDevNodeSummaryLine


 /***************************************************************************
 *
 * Function Name: PrintPnpError
 *
 * Description:   Describe the error result.
 * Parameters:    int iError
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
int PrintPnpError (int iError)
	{
	printf ("  Return Code");
	switch (iError)
		{
		case (PNPBIOS_ERR_SUCCESS):
			printf (" %02xh: Success.\n", iError); break;
		case (PNPBIOS_ERR_ACK):
			printf (" %02xh: Acknowledged.\n", iError); break;
		case (PNPBIOS_ERR_UNKNOWN_FUNCTION):
			printf (" %02xh: Unknown Function.\n", iError); break;
		case (PNPBIOS_ERR_FUNCTION_NOT_SUPPORTED):
			printf (" %02xh: Function not supported.\n", iError); break;
		case (PNPBIOS_ERR_INVALID_HANDLE):
			printf (" %02xh: Invalid handle.\n",iError); break;
		case (PNPBIOS_ERR_BAD_PARAMETER):
			printf (" %02xh: Bad parameter.\n",iError); break;
		case (PNPBIOS_ERR_SET_FAILED):
			printf (" %02xh: Set failed.\n",iError); break;
		case (PNPBIOS_ERR_EVENTS_NOT_PENDING):
			printf (" %02xh: Events not pending.\n", iError); break;
		case (PNPBIOS_ERR_SYSTEM_NOT_DOCKED):
			printf (" %02xh: System not docked.\n", iError); break;
		case (PNPBIOS_ERR_NO_ISA_PNP_CARDS):
			printf (" %02xh: No isa pnp cards.\n", iError); break;
		case (PNPBIOS_ERR_CANT_DETERMINE_DOCKING):
			printf (" %02xh: Cant determine docking.\n", iError); break;
		case (PNPBIOS_ERR_CHANGE_FAILED_NO_BATTERY):
			printf (" %02xh: Change failed no battery.\n", iError); break;
		case (PNPBIOS_ERR_CHANGE_FAILED_CONFLICT):
			printf (" %02xh: Change failed conflict.\n", iError); break;
		case (PNPBIOS_ERR_BUFFER_TOO_SMALL):
			printf (" %02xh: Buffer too small.\n", iError); break;
		case (PNPBIOS_ERR_USE_ESCD_SUPPORT):
			printf (" %02xh: Use escd support.\n", iError); break;
		default:
			printf (" %02xh: Unknown Error Code.\n", iError);
		}// end switch

	return (iError);

	} // end PrintPnpError


  /***************************************************************************
 *
 * Function Name: PrintPnpHeader
 *
 * Description:   Display the PnP Header
 * Parameters:    None.
 * Returns:       None.
 *
 * Notes:
 *
 *************************************************************************/
void PrintPnpHeader ()
	{
	printf("== $PnP Header, located at %lp ==\n", gfpPnpHeader);

	printf("    $PnP Signature       = %08lX\n",gfpPnpHeader->dSignature);
	printf("    Revision             = %02X \n",gfpPnpHeader->bRevision);
	printf("    Length               = %02X \n",gfpPnpHeader->bLength);
	printf("    Control              = %04X \n",gfpPnpHeader->wControl);
	printf("    Checksum             = %02X \n",gfpPnpHeader->bChecksum);
	printf("    EventNotify          = %08lX\n",gfpPnpHeader->dEventNotify);
	printf("    RealEntryOffset      = %04X \n",gfpPnpHeader->wRealEntryOffset);
	printf("    RealCodeSegment      = %04X \n",gfpPnpHeader->wRealCodeSegment);
	printf("    ProtectedEntryOffset = %04X \n",gfpPnpHeader->wProtectedEntryOffset);
	printf("    ProtectedCodeSegBase = %08lX\n",gfpPnpHeader->dProtectedCodeSegBase);
	printf("    OemDeviceId          = %08lX\n",gfpPnpHeader->dOemDeviceId);
	printf("    RealDataSegment      = %04X \n",gfpPnpHeader->wRealDataSegment);
	printf("    ProtectedDataSegBase = %08lX\n",gfpPnpHeader->dProtectedDataSegBase);

	} // end PrintPnpHeader


/* END OF FILE - PNPBIOS.CPP */
