 /*
 *
 * TITLE   PNPBIOS.H  -  PnP BIOS Demonstration Program.
 *
 * Copyright (c) 1995 by Glenn Jystad, All Rights Reserved.
 *
 ***************************************************************************
 *
 * Header Filename:  PNPBIOS.H
 *
 **************************************************************************/

#ifndef PNPBIOS_H
#define PNPBIOS_H

#pragma warning(disable:4103)

 /*************************************************************************
 *                    Defines/Constants
 **************************************************************************/

#define  PNP_SIG_SEARCH_BASE   					0xF000
#define  PNP_DW_SIGNATURE      					0x506E5024  // $PnP

// Function Numbers

#define  PNPBIOS_GETNODEINFO                    0x00
#define  PNPBIOS_GETNODE                        0x01
#define  PNPBIOS_SETNODE                        0x02
#define  PNPBIOS_GETEVENT                       0x03
#define  PNPBIOS_SENDMESSAGE                    0x04
#define  PNPBIOS_GETDOCKINFO                    0x05

#define  PNPBIOS_SETSTATICRES                   0x09
#define  PNPBIOS_GETSTATICRES                   0x0A
#define  PNPBIOS_GETAPMTABLE                    0x0B

#define  PNPBIOS_GETISACONFIG                   0x40
#define  PNPBIOS_GETESCDINFO                    0x41
#define  PNPBIOS_GETESCD                        0x42
#define  PNPBIOS_SETESCD                        0x43

// Error Codes

#define  PNPBIOS_ERR_SUCCESS					0x00
#define  PNPBIOS_ERR_ACK                        0x01
#define  PNPBIOS_ERR_UNKNOWN_FUNCTION			0x81
#define  PNPBIOS_ERR_FUNCTION_NOT_SUPPORTED		0x82
#define  PNPBIOS_ERR_INVALID_HANDLE				0x83
#define  PNPBIOS_ERR_BAD_PARAMETER				0x84
#define  PNPBIOS_ERR_SET_FAILED					0x85
#define  PNPBIOS_ERR_EVENTS_NOT_PENDING			0x86
#define  PNPBIOS_ERR_SYSTEM_NOT_DOCKED			0x87
#define  PNPBIOS_ERR_NO_ISA_PNP_CARDS			0x88
#define  PNPBIOS_ERR_CANT_DETERMINE_DOCKING		0x89
#define  PNPBIOS_ERR_CHANGE_FAILED_NO_BATTERY	0x8A
#define  PNPBIOS_ERR_CHANGE_FAILED_CONFLICT		0x8B
#define  PNPBIOS_ERR_BUFFER_TOO_SMALL			0x8C
#define  PNPBIOS_ERR_USE_ESCD_SUPPORT			0x8D

// Parameter Masks and Values

#define  CONTROL_EVENT_NOTIFY_MASK              0x3
#define  CONTROL_EVENT_NOTIFY_SHIFT             0x0

#define  SETNODE_UPDATECURRENT                  0x1
#define  SETNODE_UPDATENEXTBOOT                 0x2
#define  SETNODE_UPDATEALL                      0x3

// Device Node Attributes

#define  DEV_ATTR_CONFIG_MASK              		0x0180 // bit MASK
#define  DEV_ATTR_DYNAONLY                 		0x0180
#define  DEV_ATTR_STATONLY                 		0x0000 // bits 7,8 off
#define  DEV_ATTR_DYNASTAT                 		0x0080
#define  DEV_ATTR_REMOVEABLE               		0x0040
#define  DEV_ATTR_DOCKING                  		0x0020
#define  DEV_ATTR_IPL                      		0x0010
#define  DEV_ATTR_PRI_INPUT                		0x0008
#define  DEV_ATTR_PRI_OUTPUT               		0x0004
#define  DEV_ATTR_CONFIGURABLE             		0x0002
#define  DEV_ATTR_DISABLABLE               		0x0001

#ifndef BYTE
typedef  unsigned char  BYTE;
typedef  unsigned short WORD;
typedef  unsigned long  DWORD;
#endif

 /*************************************************************************
 *                    Classes/Structures
 **************************************************************************/


// MUST have pragma pack(1)

#pragma pack(1)
typedef struct PnpHeader
{
   DWORD    dSignature;
   BYTE     bRevision;
   BYTE     bLength;
   WORD     wControl;
   BYTE     bChecksum;
   DWORD    dEventNotify;
   WORD     wRealEntryOffset;
   WORD     wRealCodeSegment;
   WORD     wProtectedEntryOffset;
   DWORD    dProtectedCodeSegBase;
   DWORD    dOemDeviceId;
   WORD     wRealDataSegment;
   DWORD    dProtectedDataSegBase;
} PnpHeader;

typedef struct DockingStationInfo
{
	DWORD		dwDSLocID;
	DWORD		dwSerialNo;
	WORD		wCapabilities;
} DockingStationInfo;

typedef struct IsaConfig
{
	BYTE		bRevision;
	BYTE		bNumCsnAssigned;
	WORD		wIsaReadDataPort;
	WORD		wReserved;
} IsaConfig;


 /*************************************************************************
 *                    Function Prototypes
 **************************************************************************/

   /* ----------------------------------------------------------------------
      The following functions are declared as static union so that they
      can all share the same value as the specified entry point.
      This way only fpVoid need be set to the seg:ofs from the
      PnpHeader structure for the Read Mode entry point.
   */
typedef union
      {
      void far * fpVoid;
      void far * pnpRealModeEntry;

      // Get Number of System Device Nodes
      int   (far *fpfnPnpFn0)(WORD      wFunction,
                              WORD far *pwNumNodes,
                              WORD far *pwNodeSize,
                              WORD      wBiosSelector);

      // Get System Device Node
      int   (far *fpfnPnpFn1)(WORD      wFunction,
                              WORD far *pwNode,
                              BYTE far *pbyDevNodeBuffer,
                              WORD      wControl,
                              WORD      wBiosSelector);

      // Set System Device Node
      int   (far *fpfnPnpFn2)(WORD      wFunction,
                              BYTE      byNode,
                              BYTE far *pbyDevNodeBuffer,
                              WORD      wControl,
                              WORD      wBiosSelector);

      // Get Event
      int   (far *fpfnPnpFn3)(WORD      wFunction,
                              WORD far *pwMessage,
                              WORD      wBiosSelector);

      // Send Message
      int   (far *fpfnPnpFn4)(WORD wFunction,
                              WORD wMessage,
                              WORD wBiosSelector);

      // Get Docking Station Information
      int   (far *fpfnPnpFn5)(WORD                    wFunction,
                              DockingStationInfo far *pdsInfo,
                              WORD                    wBiosSelector);

      // Set Statically Allocated Resource Information
      int   (far *fpfnPnpFn9)(WORD      wFunction,
                              BYTE far *fpResBlk,
                              WORD      wBiosSelector);

      // Get Statically Allocated Resource Information
      int   (far *fpfnPnpFnA)(WORD      wFunction,
                              BYTE far *fpResBlk,
                              WORD      wBiosSelector);

      // Get APM 1.1 ID Table
      int   (far *fpfnPnpFnB)(WORD      wFunction,
                              WORD far *fpwSize,
                              BYTE far *fpApmIdTable,
                              WORD      wBiosSelector);

      // Get ISA Configuration Structure
      int   (far *fpfnPnpFn40)(WORD          wFunction,
                              IsaConfig far *picInfo,
                              WORD           wBiosSelector);

      // Get ESCD Information
      int   (far *fpfnPnpFn41)(WORD       wFunction,
                               WORD far  *fpwNvramSize,
                               WORD far  *fpwEscdSize,
                               DWORD far *fpwNvramBase,
                               WORD       wBiosSelector);
      // Read ESCD
      int   (far *fpfnPnpFn42)(WORD      wFunction,
                               BYTE far *fpbyEscdBuf,
                               WORD      wEscdSelector,
                               WORD      wBiosSelector);
      // Write ESCD
      int   (far *fpfnPnpFn43)(WORD      wFunction,
                               BYTE far *fpbyEscdBuf,
                               WORD      wEscdSelector,
                               WORD      wBiosSelector);

      } PnpFunctions;


	// Find $PnP Header and Save it.
	int PnpGetHeader();
	char * EisaId (DWORD dwCompressedID, char * pszBuffer);

 /*************************************************************************
 *                    Global Variables
 **************************************************************************/
#ifdef PNPBIOS_CPP

 PnpHeader far * gfpPnpHeader;
 PnpFunctions	 guPnpFunctions;
 WORD			 gwBiosSelector;

#else

// Definitions made public by PNPBIOS.CPP

 extern PnpHeader far * gfpPnpHeader;
 extern WORD		 	gwBiosSelector;
 extern PnpFunctions	guPnpFunctions;

 extern	void DumpDevNode (BYTE * pbDevNode);
 extern	void DumpDevNodeSummaryLine (BYTE * pbDevNode);
 int PrintPnpError (int iError);
 void PrintPnpHeader (void);
 extern void DumpDevNodeSummary (void);

#endif

#endif
/* END OF FILE - PNPBIOS.H */
