 /*
 *
 * TITLE   PNPDEMO.CPP  -  PnP BIOS Demonstration Program.
 *
 * Copyright (c) 1995 by Glenn Jystad, All rights reserved.
 *
 ***************************************************************************
 *
 * Filename:         PNPDEMO.CPP
 *
 **************************************************************************/

#define PNPDEMO_CPP

 /*************************************************************************
 *                    Include Files
 **************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "STDDEFS.H"
#include "PNPBIOS.H"
#include "PNPISA.H"
#include "UTILS.H"

#pragma warning(disable:4100)

 /*************************************************************************
 *                    Local Defines
 **************************************************************************/

#define		CFG_DATA_SIZE	4096
 /*************************************************************************
 *                    Global Variable Declarations
 **************************************************************************/

	int 	giMajor = 1;
	int 	giMinor = 0;

	BYTE	gabCfgData[CFG_DATA_SIZE];		// Global data buffer.

 /*************************************************************************
 *                    Functions
 **************************************************************************/


  /***************************************************************************
 *
 * Function Name: PrintTitle
 *
 * Description:   Display the program's title, version, and copyright.
 * Parameters:    None.
 * Returns:       None.
 *
 * Notes:
 *
 *************************************************************************/
void PrintTitle ()
	{
	printf ("PnPDemo V%d.%02d - Copyright 1995 Glenn Jystad.\n\n",
	        giMajor, giMinor);
	} // end PrintTitle


  /***************************************************************************
 *
 * Function Name: PrintUsage
 *
 * Description:   Display command line PrintUsage.
 * Parameters:    None.
 * Returns:       None.
 *
 * Notes:
 *
 *************************************************************************/
void PrintUsage ()
	{
	printf ("Usage:	pnpdemo [-F#,H,S,?]\n");
	printf ("          -F# : Execute PnP BIOS Function number '#'\n");
	printf ("                -F0\n");
	printf ("                -F1 node Current/Next\n");
	printf ("                -F2 node Current/Next/Both\n");
	printf ("                -F3\n");
	printf ("                -F4 message\n");
	printf ("                -F5\n");
	printf ("                -F9\n");
	printf ("                -FA\n");
	printf ("                -FB\n");
	printf ("                -F40\n");
	printf ("                -F41\n");
	printf ("                -F42\n");
	printf ("                -F43\n");
	printf ("          -H  : Display PnP Header.\n");
	printf ("          -S  : Display Device Node Resource Summary.\n");
	printf ("          -?  : Display this usage message.\n");
	printf ("\n");
	printf ("                Default uses the Real Mode Entry Point. \n");
	} // end PrintUsage


 /***************************************************************************
 *
 * Function Name: PnPFnGetNodeInfo
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetNodeInfo(int argc, char * argv[])
	{
	int 	iReturn = 0;
	WORD    wNumNodes = 0;
	WORD	wNodeSize = 0;

	printf("== Pnp Function 00h, Get Node Info ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn0)(PNPBIOS_GETNODEINFO,
                            (WORD far *)&wNumNodes,
                            (WORD far *)&wNodeSize,
                            gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		printf ("    Number of nodes = %d.\n", wNumNodes);
		printf ("    Largest nodes   = %d bytes.\n", wNodeSize);
		}// end if

	return (iReturn);

	} // end PnPFnGetNodeInfo


 /***************************************************************************
 *
 * Function Name: PnPFnGetNode
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetNode(int argc, char * argv[])
	{
	int 	iReturn = 0;
	WORD  	wNode 	= atoi(argv[2]);
	WORD 	wControl = 1;			// Default to Current.


	// If Current/Next boot provided, get it.

	if (argc > 2)
		{
		wControl = (tolower(argv[3][0]) == 'n' ? 2 : 1);
		}// end if


	printf("== Pnp Function 01h, Get Device Node %d, %s ==\n", wNode,
			(wControl == 2 ? "Next Boot Settings" : "Current Settings"));

    iReturn = (*guPnpFunctions.fpfnPnpFn1)(PNPBIOS_GETNODE,
				                           (WORD far *)&wNode,
				                           gabCfgData,
				                           wControl,
				                           gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		printf ("    Next node = %d.\n", wNode);
		DumpDevNode (gabCfgData);
		printf ("\n  Binary Output:\n");
		PrintDebugOutput (gabCfgData, *(WORD *)(gabCfgData));
		}// end if

	return (iReturn);

	} // end PnPFnGetNode


 /***************************************************************************
 *
 * Function Name: PnPFnSetNode
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnSetNode(int argc, char * argv[])
	{
	int iReturn = 0;

	printf("== Pnp Function 02h, Set Device Node ==\n");

	puts ("*** Function not supported within this PnP DEMO. ***");

	return (iReturn);

	} // end PnPFnSetNode


 /***************************************************************************
 *
 * Function Name: PnPFnGetEvent
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetEvent(int argc, char * argv[])
	{
	int iReturn = 0;
	WORD wMessage;

	printf("== Pnp Function 03h, Get Event ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn3)(PNPBIOS_GETEVENT,
				                           (WORD far *)&wMessage,
				                           gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		printf ("\tEvent = %d.\n", wMessage);
		}// end if

	return (iReturn);

	} // end PnPFnGetEvent


 /***************************************************************************
 *
 * Function Name: PnPFnSendMessage
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnSendMessage(int argc, char * argv[])
	{
	int iReturn = 0;
	WORD wMessage = 0;

	printf("== Pnp Function 04h, Send Message ==\n");

	puts ("*** Need input from arguments. ***");

    iReturn = (*guPnpFunctions.fpfnPnpFn4)(PNPBIOS_SENDMESSAGE,
				                           wMessage,
				                           gwBiosSelector);

	PrintPnpError(iReturn);

	return (iReturn);

	} // end PnPFnSendMessage


 /***************************************************************************
 *
 * Function Name: PnPFnGetDockInfo
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetDockInfo(int argc, char * argv[])
	{
	int iReturn = 0;
	DockingStationInfo dsInfo;

	printf("== Pnp Function 05h, Get Docking Station Information ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn5)(PNPBIOS_GETDOCKINFO,
										   (DockingStationInfo far *)&dsInfo,
				                           gwBiosSelector);
	if (!PrintPnpError(iReturn))
		{
		printf ("\tStation Id	 = %08lX\n", dsInfo.dwDSLocID);
		printf ("\tSerial Number = %08lX\n", dsInfo.dwSerialNo);
		printf ("\tCapabilities  = %04X\n", dsInfo.wCapabilities);
		}// end if

	return (iReturn);

	} // end PnPFnGetDockInfo


 /***************************************************************************
 *
 * Function Name: PnPFnSetStaticResources
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnSetStaticResources(int argc, char * argv[])
	{
	int iReturn = 0;

	printf("== Pnp Function 09h, Set Statically Allocated Resources ==\n");

	puts ("*** Function not supported within this PnP DEMO. ***");

	return (iReturn);

	} // end PnPFnSetStaticResources


 /***************************************************************************
  *
 * Function Name: PnPFnGetStaticResources
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetStaticResources(int argc, char * argv[])
	{
	int 		iReturn = 0;

	printf("== Pnp Function 0Ah, Get Statically Allocated Resources ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFnA)(PNPBIOS_GETSTATICRES,
                            			   gabCfgData,
			                               gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		BYTE *pbIsa = gabCfgData;
		DumpIsaSumResources(&pbIsa, SUM_LONG_FORM);
		//printf ("\n  Binary Output:\n");
		//PrintDebugOutput (gabCfgData, 0x200);
		}// end if

	return (iReturn);

	} // end PnPFnGetStaticResources


 /***************************************************************************
 *
 * Function Name: PnPFnGetApmTable
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetApmTable(int argc, char * argv[])
	{
	int iReturn = 0;
	WORD wSize = CFG_DATA_SIZE;

	printf("== Pnp Function 0Bh, Get APM 1.1 Table ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFnB)(PNPBIOS_GETAPMTABLE,
                              			   (WORD far *)&wSize,
                              			   (BYTE far *)gabCfgData,
			                               gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		PrintDebugOutput (gabCfgData, 0x200);
		}// end if

	return (iReturn);

	} // end PnPFnGetApmTable


 /***************************************************************************
 *
 * Function Name: PnPFnGetIsaConfig
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetIsaConfig(int argc, char * argv[])
	{
	int iReturn = 0;
	IsaConfig 	isaInfo;

	printf("== Pnp Function 40h, Get PnP ISA Configuration ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn40)(PNPBIOS_GETISACONFIG,
                              				&isaInfo,
			                                gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		printf ("\tRevision	 	  = %02Xh\n", isaInfo.bRevision);
		printf ("\tMax CSN	 	  = %d\n", isaInfo.bNumCsnAssigned);
		printf ("\tRead Data Port = %04Xh\n", isaInfo.wIsaReadDataPort);
		}// end if

	return (iReturn);

	} // end PnPFnGetIsaConfig


 /***************************************************************************
 *
 * Function Name: PnPFnGetEscdInfo
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetEscdInfo(int argc, char * argv[])
	{
	int 	iReturn = 0;
    WORD 	wNvramSize = 0;
    WORD 	wEscdSize = 0;
    DWORD 	dNvramBase = 0;

	printf("== Pnp Function 41h, Get Escd Information ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn41)(PNPBIOS_GETESCDINFO,
                               				&wNvramSize,
			                                &wEscdSize,
            			                    &dNvramBase,
			                                gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		printf ("\tNVRAM Size = %u\n", wNvramSize);
		printf ("\tESCD Size  = %u\n", wEscdSize);
		printf ("\tNvram Base = %08lXh\n", dNvramBase);
		}// end if

	return (iReturn);

	} // end PnPFnGetEscdInfo


 /***************************************************************************
 *
 * Function Name: PnPFnGetEscd
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnGetEscd(int argc, char * argv[])
	{
	int iReturn = 0;
    WORD      wEscdSelector = 0;

	printf("== Pnp Function 42h, Get Escd Data ==\n");

    iReturn = (*guPnpFunctions.fpfnPnpFn42)(PNPBIOS_GETESCD,
							   				gabCfgData,
                               				wEscdSelector,
			                                gwBiosSelector);

	if (!PrintPnpError(iReturn))
		{
		PrintDebugOutput (gabCfgData, *(WORD *)(gabCfgData));
		}// end if


	return (iReturn);

	} // end PnPFnGetEscd


 /***************************************************************************
 *
 * Function Name: PnPFnSetEscd
 *
 * Description:
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnPFnSetEscd(int argc, char * argv[])
	{
	int iReturn = 0;

	printf("== Pnp Function 43h, Set Escd Data ==\n");

	puts ("*** Function not supported within this PnP DEMO. ***");

	return (iReturn);

	} // end PnPFnSetEscd


 /***************************************************************************
 *
 * Function Name: PnpFunction
 *
 * Description:   Execute the specified Pnp Function
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnpFunction (int argc, char* argv[])
	{
	int iReturn = 0;
	DWORD dFn = strtoul(&argv[1][2], NULL, 16);

	if (dFn == PNPBIOS_GETNODEINFO)
		{
		iReturn = PnPFnGetNodeInfo(argc, argv);
		}// end if
	else if (dFn == PNPBIOS_GETNODE)
		{
		iReturn = PnPFnGetNode(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_SETNODE)
		{
		iReturn = PnPFnSetNode(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETEVENT)
		{
		iReturn = PnPFnGetEvent(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_SENDMESSAGE)
		{
		iReturn = PnPFnSendMessage(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETDOCKINFO)
		{
		iReturn = PnPFnGetDockInfo(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_SETSTATICRES)
		{
		iReturn = PnPFnSetStaticResources(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETSTATICRES)
		{
		iReturn = PnPFnGetStaticResources(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETAPMTABLE)
		{
		iReturn = PnPFnGetApmTable(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETISACONFIG)
		{
		iReturn = PnPFnGetIsaConfig(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETESCDINFO)
		{
		iReturn = PnPFnGetEscdInfo(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_GETESCD)
		{
		iReturn = PnPFnGetEscd(argc, argv);
		}// end else
	else if (dFn == PNPBIOS_SETESCD)
		{
		iReturn = PnPFnSetEscd(argc, argv);
		}// end else
	else
		{
		printf("  %02lXh if not a supported PnP BIOS function number.\n", dFn);
		}// end else

	return (iReturn);

	} // end PnpFunction


 /***************************************************************************
 *
 * Function Name: PnpSummary
 *
 * Description:   Display a summary of the device nodes.
 * Parameters:    int argv, char** argv
 * Returns:       exit code
 *
 * Notes:
 *
 *************************************************************************/
int PnpSummary (int argc, char* argv[])
	{
	int iReturn = 0;

	printf("== PnP Summary ==\n");

	// Print summary on all device nodes.
	DumpDevNodeSummary();

	return (iReturn);

	} // end PnpSummary


 /***************************************************************************
 *
 * Function Name: PnpDemo
 *
 * Description:   PnP Demo
 * Parameters:    argc = count of arguments, argv = array of strings
 * Returns:       int - Dos Exit Code
 *
 * Notes:
 *
 *************************************************************************/
int PnpDemo (int argc, char *argv[])
	{
	int iReturn=0;

	if (argc < 1)
		{
		PrintUsage();
		}// end if
	else
		{
		// Determine if the current BIOS contains a PnP Header.
		if (PnpGetHeader())
			{
 			printf ("This BIOS does not contain a valid PnP header.\n");
			exit(1);
			}// end if

		switch (toupper(argv[1][1]))
			{
			case ('F'):
				iReturn = PnpFunction(argc, argv);
				break;

			case ('H'):
				PrintPnpHeader();
				break;

			case ('S'):
				iReturn = PnpSummary(argc, argv);
				break;

			case ('?'):
			default:
				PrintUsage();
			}// end switch

		}// end else

	return (iReturn);
	} // end PnpDemo


 /***************************************************************************
 *
 * Function Name: main
 *
 * Description:   PnP Demo
 * Parameters:    argc = count of arguments
 * Returns:       int - Dos Exit Code
 *
 * Notes:
 *
 *************************************************************************/
void main (int argc, char *argv[])
	{
	int iReturn=0;

	PrintTitle();
	PnpDemo(argc, argv);
	exit (iReturn);

	} // end main


/* END OF FILE - PNPDEMO.CPP */
