 /*
 *
 * TITLE   PNPISA.CPP  -  PnP ISA Data Type Parsing.
 *
 * Copyright (c) 1995 by Glenn Jystad, All Rights Reserved.
 *
 ***************************************************************************
 *
 * Filename:         PNPISA.CPP
 *
 **************************************************************************/

#define PNPISA_CPP

 /*************************************************************************
 *                    Include Files
 **************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "STDDEFS.H"
#include "PNPISA.H"
#include "UTILS.H"

 /*************************************************************************
 *                    Local Defines
 **************************************************************************/

 /*************************************************************************
 *                    Global Variable Declarations
 **************************************************************************/

 /*************************************************************************
 *                    Functions
 **************************************************************************/


 /***************************************************************************
 *
 * Function Name: DumpIsaResources
 *
 * Description:   Display Isa Resources until END_TAG
 * Parameters:    BYTE **ppbIsa
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void DumpIsaResources (BYTE **ppbIsa)
	{
	int	fEndTagFound = FALSE;
	do
		{
		if (((IsaRes*)(*ppbIsa))->ver.tag.bTagType)
			{
			// Large Resource Items
			switch (((IsaRes*)(*ppbIsa))->mem.tag.bLargeTagField)
				{
				case (ISA_MEMORY):
					{
					IsaResMemory* pMem = (IsaResMemory*) *ppbIsa;
					printf ("    Memory:\n");
					printf ("        Info    - %02Xh\n", pMem->bMemInfo);
					printf ("        Base Min- %08lXh\n", (DWORD)(pMem->wMemBaseMin) * 0x100);
					printf ("        Base Max- %08lXh\n", (DWORD)(pMem->wMemBaseMax) * 0x100);
					printf ("        Align   - %08lXh\n", (pMem->wMemAlignment ==  0 ?
	                							  0x10000 :
	                                              (DWORD)(pMem->wMemAlignment) * 0x100));
					printf ("        Range   - %08lXh\n", (DWORD)(pMem->wMemRange) * 0x100);
					}
					break;

				case (ISA_ANSI_STR):
					{
					IsaResAnsiString* pAnsi = (IsaResAnsiString*)*ppbIsa;
					printf ("    Ansi String:\n");
					printf ("        String  - ");
					DumpChars(pAnsi->tag.wLength,&(pAnsi->bData));
					printf ("\n");
					}
					break;

				case (ISA_UNICODE_STR):
					{
					IsaResUnicodeString* pUni= (IsaResUnicodeString*)*ppbIsa;
					printf ("    Unicode String:\n");
					printf ("        Country  - %02Xh\n", pUni->wCountry);
					printf ("        Unicode  - ");
					DumpChars(pUni->tag.wLength - 2,&(pUni->bData));
					printf ("\n");
					}
					break;

				case (ISA_MEMORY_32):
					{
					IsaResMem32* pMem32 = (IsaResMem32*) *ppbIsa;
					printf ("    32-Bit Memory:\n");
					printf ("        Info     - %02Xh\n", pMem32->bMemInfo);
					printf ("        Base Min - %08lXh\n", pMem32->dwMemBaseMin);
					printf ("        Base Max - %08lXh\n", pMem32->dwMemBaseMax);
					printf ("        Align    - %08lXh\n", pMem32->dwMemAlignment);
					printf ("        Range    - %08lXh\n", pMem32->dwMemRange);
					}
					break;

				case (ISA_MEMORY_32_FIXED):
					{
					IsaResMem32Fixed* pMem32F = (IsaResMem32Fixed*) *ppbIsa;
					printf ("    Fixed 32-Bit Memory:\n");
					printf ("        Info     - %02Xh\n", pMem32F->bMemInfo);
					printf ("        Base     - %08lXh\n", pMem32F->dwMemBase);
					printf ("        Range    - %08lXh\n", pMem32F->dwMemRange);
					}
					break;

				//default:

				}// end switch
			(*ppbIsa) += ((IsaRes*)(*ppbIsa))->mem.tag.wLength;
			(*ppbIsa) += 3;		// Account for TAG byte and Length word too.
			}// end if
		else
			{
			// Small Resource Items
			switch (((IsaRes*)(*ppbIsa))->ver.tag.bSmallTagField)
				{
				case (ISA_PNP_VERSION):
					{
					IsaResPnPVersion* pVer = (IsaResPnPVersion*)*ppbIsa;
					printf ("    PnP Version:\n");
					printf ("        Version  - %02Xh\n", pVer->bVersion);
					printf ("        Vendor   - %02Xh\n", pVer->bVendor);
					}
					break;

				case (ISA_LOGICAL_ID):
					{
					char acBuf[10];
					IsaResLogicalId* pId = (IsaResLogicalId*)*ppbIsa;
					printf ("    Logical Id:\n");
					printf ("        ID       - %s\n", EisaId(*(DWORD*)(&(pId->dwId)), acBuf));
					printf ("        FlagsA   - %02Xh\n", pId->bFlags5);
					printf ("        FlagsB   - %02Xh\n", pId->bFlags6);
					}
					break;

				case (ISA_COMPAT_ID):
					{
					char acBuf[10];
					IsaResCompatId* pId = (IsaResCompatId*)*ppbIsa;
					printf ("    Compatible Id:\n");
					printf ("        ID       - %s\n", EisaId(*(DWORD*)(&(pId->dwId)), acBuf));
					}
					break;

				case (ISA_IRQ):
					{
					IsaResIrq* pIrq = (IsaResIrq*)*ppbIsa;
					printf ("    Irq:\n");
					printf ("        Mask     - %04Xh\n", pIrq->wIrqMask);
					printf ("        Info     - %02Xh\n", pIrq->bIrqInfo);
					}
					break;

				case (ISA_DMA):
					{
					IsaResDma* pDma = (IsaResDma*)*ppbIsa;
					printf ("    Dma:\n");
					printf ("        Mask     - %02Xh\n", pDma->bDmaMask);
					printf ("        Info     - %02Xh\n", pDma->bDmaInfo);
					}
					break;

				case (ISA_START_DEP):
					{
					IsaResStartDep* pSDF = (IsaResStartDep*)*ppbIsa;
					printf ("    Start Dependent Function:\n");
					printf ("        Priority - %02Xh\n", pSDF->bPriority);
					}
					break;

				case (ISA_END_DEP):
					printf ("    End Dependent Function:\n");
					break;

				case (ISA_IO_PORT):
					{
					IsaResIo* pIo = (IsaResIo*)*ppbIsa;
					printf ("    Io:\n");
					printf ("        Info     - %02Xh\n", pIo->bIoInfo);
					printf ("        Base Min - %04Xh\n", pIo->wIoBaseMin);
					printf ("        Base Max - %04Xh\n", pIo->wIoBaseMax);
					printf ("        Align    - %02Xh\n", pIo->bIoAlignment);
					printf ("        Range    - %u\n", pIo->bIoRange);
					}
	                break;

				case (ISA_FIXED_IO):
					{
					IsaResFixedIo* pFIo = (IsaResFixedIo*)*ppbIsa;
					printf ("    Fixed Io:\n");
					printf ("        Base     - %04Xh\n", pFIo->wIoBase);
					printf ("        Range    - %u\n", pFIo->bIoRange);
					}
	                break;

				case (ISA_END_TAG):
					{
					IsaResEnd* pEnd = (IsaResEnd*)*ppbIsa;
					printf ("    End Tag:\n");
					printf ("        Checksum - %02Xh\n", pEnd->bChecksum);
					fEndTagFound = TRUE;
					}
					break;

				//default:

				}// end switch
			(*ppbIsa) += ((IsaRes*)(*ppbIsa))->ver.tag.bSmallTagLength;
			(*ppbIsa) ++;		// Account for TAG Byte too.
			}// end else

		} while (!fEndTagFound);

	} // end DumpIsaResources

 /***************************************************************************
 *
 * Function Name: DumpIsaSumResources
 *
 * Description:   Display Isa Resources until END_TAG
 * Parameters:    BYTE **ppbIsa - Ptr to resources
 *                int iLongSum  - 0 = Short, 1 = Long form.
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void DumpIsaSumResources (BYTE **ppbIsa, int iLongSum)
	{
	int	fEndTagFound = FALSE;
	do
		{
		if (((IsaRes*)(*ppbIsa))->ver.tag.bTagType)
			{
			// Large Resource Items
			switch (((IsaRes*)(*ppbIsa))->mem.tag.bLargeTagField)
				{
				case (ISA_MEMORY):
					{
					IsaResMemory* pMem = (IsaResMemory*) *ppbIsa;
					printf ("%06lXh - %06lXh, ",
							(DWORD)(pMem->wMemBaseMin) * 0x100,
							((DWORD)(pMem->wMemBaseMin) * 0x100) +
							((DWORD)(pMem->wMemRange) * 0x100) - 1);
					}
					break;

				case (ISA_ANSI_STR):
					{
					//IsaResAnsiString* pAnsi = (IsaResAnsiString*)*ppbIsa;
					//printf ("    Ansi    - ");
					//DumpChars(pAnsi->tag.wLength,&(pAnsi->bData));
					//printf ("\n");
					}
					break;

				case (ISA_UNICODE_STR):
					{
					//IsaResUnicodeString* pUni= (IsaResUnicodeString*)*ppbIsa;
					//printf ("    Unicode -");
					//DumpChars(pUni->tag.wLength - 2,&(pUni->bData));
					//printf ("\n");
					}
					break;

				case (ISA_MEMORY_32):
					{
					IsaResMem32* pMem32 = (IsaResMem32*) *ppbIsa;
					printf ("%08lXh - %08lXh, ",
							pMem32->dwMemBaseMin,
							pMem32->dwMemBaseMin+
							pMem32->dwMemRange - 1);
					}
					break;

				case (ISA_MEMORY_32_FIXED):
					{
					IsaResMem32Fixed* pMem32F = (IsaResMem32Fixed*) *ppbIsa;
					printf ("%08lXh - %08lXh, ",
							pMem32F->dwMemBase,
							pMem32F->dwMemBase+
							pMem32F->dwMemRange - 1);
					}
					break;

				//default:

				}// end switch
			(*ppbIsa) += ((IsaRes*)(*ppbIsa))->mem.tag.wLength;
			(*ppbIsa) += 3;		// Account for TAG byte and Length word too.
			}// end if
		else
			{
			// Small Resource Items
			switch (((IsaRes*)(*ppbIsa))->ver.tag.bSmallTagField)
				{
				case (ISA_PNP_VERSION):
					{
					IsaResPnPVersion* pVer = (IsaResPnPVersion*)*ppbIsa;
					//printf ("    Version - %02Xh\n", pVer->bVersion);
					}
					break;

				case (ISA_LOGICAL_ID):
					{
					char acBuf[10];
					IsaResLogicalId* pId = (IsaResLogicalId*)*ppbIsa;
					//printf ("    Log Id  - %s\n", EisaId(*(DWORD*)(&(pId->dwId)), acBuf));
					}
					break;

				case (ISA_COMPAT_ID):
					{
					char acBuf[10];
					IsaResCompatId* pId = (IsaResCompatId*)*ppbIsa;
					//printf ("    Comp Id - %s\n", EisaId(*(DWORD*)(&(pId->dwId)), acBuf));
					}
					break;

				case (ISA_IRQ):
					{
					int iIrq;
					IsaResIrq* pIrq = (IsaResIrq*)*ppbIsa;
					while (iIrq = GetAndClearBit(&(pIrq->wIrqMask)))
						{
						printf ("Irq%u, ", iIrq);
						}// end while
					}
					break;

				case (ISA_DMA):
					{
					int iDma;
					IsaResDma* pDma = (IsaResDma*)*ppbIsa;
					WORD wDma = (WORD)(pDma->bDmaMask);
					while (iDma = GetAndClearBit(&wDma))
						{
						printf ("Dma%u, ", iDma);
						}// end while
					}
					break;

				case (ISA_START_DEP):
					{
					IsaResStartDep* pSDF = (IsaResStartDep*)*ppbIsa;
					//printf ("    Start DF- %02Xh\n", pSDF->bPriority);
					}
					break;

				case (ISA_END_DEP):
					//printf ("    End DF\n");
					break;

				case (ISA_IO_PORT):
					{
					IsaResIo* pIo = (IsaResIo*)*ppbIsa;
					if (pIo->bIoRange > 1)
						{
						printf ("%04Xh - %04Xh, ",
										pIo->wIoBaseMin,
										pIo->wIoBaseMin +
										pIo->bIoRange - 1);
						}// end if
					else
						{
						printf ("%04Xh, ", pIo->wIoBaseMin);
						}// end else
					}
	                break;

				case (ISA_FIXED_IO):
					{
					IsaResFixedIo* pFIo = (IsaResFixedIo*)*ppbIsa;
					printf ("%03Xh - %03Xh, ",
									pFIo->wIoBase,
									pFIo->wIoBase +
									pFIo->bIoRange - 1);
					}
	                break;

				case (ISA_END_TAG):
					{
					IsaResEnd* pEnd = (IsaResEnd*)*ppbIsa;
					printf ("\n");
					fEndTagFound = TRUE;
					}
					break;

				//default:

				}// end switch
			(*ppbIsa) += ((IsaRes*)(*ppbIsa))->ver.tag.bSmallTagLength;
			(*ppbIsa) ++;		// Account for TAG Byte too.
			}// end else

		if (iLongSum == SUM_LONG_FORM)
			{
			printf ("\n");
			}// end if

		} while (!fEndTagFound);

	} // end DumpIsaSumResources

/* END OF FILE - PNPISA.CPP */
