 /*
 *
 * TITLE   PNPISA.H  -  PnP ISA Data Types
 *
 * Copyright (c) 1995 by Glenn Jystad, All Rights Reserved.
 *
 ***************************************************************************
 *
 * Header Filename:  PNPISA.H
 *
 **************************************************************************/

#ifndef PNPISA_H
#define PNPISA_H

#pragma pack(1)

 /*************************************************************************
 *                    Defines/Constants
 **************************************************************************/

#define		SUM_SHORT_FORM		0
#define		SUM_LONG_FORM		1


/* Small Resource Items */

#define		ISA_PNP_VERSION		0x01
#define		ISA_LOGICAL_ID		0x02
#define		ISA_COMPAT_ID		0x03
#define		ISA_IRQ				0x04
#define		ISA_DMA				0x05
#define		ISA_START_DEP		0x06
#define		ISA_END_DEP			0x07
#define		ISA_IO_PORT			0x08
#define		ISA_FIXED_IO		0x09
#define		ISA_END_TAG			0x0F

/* Large Resource Items */

#define		ISA_MEMORY			0x01
#define		ISA_ANSI_STR		0x02
#define		ISA_UNICODE_STR		0x03
#define		ISA_MEMORY_32		0x05
#define		ISA_MEMORY_32_FIXED	0x06

 /*************************************************************************
 *                    Classes/Structures
 **************************************************************************/

/* Small Resource Items */

typedef struct
	{
	BYTE	bSmallTagLength : 3;	// Bits 2:0 - Length - Tag byte.
	BYTE	bSmallTagField 	: 4;	// Bits 6:3 - Small Tag Type
	BYTE	bTagType 		: 1;	// Bit 7	- Small = 0, Large = 1
	} IsaSmallTag;

typedef struct
	{
	IsaSmallTag tag;
	BYTE		bVersion;
	BYTE		bVendor;
	} IsaResPnPVersion;

typedef struct
	{
	IsaSmallTag tag;
	DWORD		dwId;
	BYTE		bFlags5;
	BYTE		bFlags6;
	} IsaResLogicalId;

typedef struct
	{
	IsaSmallTag tag;
	DWORD		dwId;
	} IsaResCompatId;

typedef struct
	{
	IsaSmallTag tag;
	WORD		wIrqMask;
	BYTE		bIrqInfo;
	} IsaResIrq;

typedef struct
	{
	IsaSmallTag tag;
	BYTE		bDmaMask;
	BYTE		bDmaInfo;
	} IsaResDma;

typedef struct
	{
	IsaSmallTag tag;
	BYTE		bPriority;
	} IsaResStartDep;

typedef struct
	{
	IsaSmallTag tag;
	} IsaResEndDep;

typedef struct
	{
	IsaSmallTag tag;
	BYTE		bIoInfo;
	WORD		wIoBaseMin;
	WORD		wIoBaseMax;
	BYTE		bIoAlignment;
	BYTE		bIoRange;
	} IsaResIo;

typedef struct
	{
	IsaSmallTag tag;
	WORD		wIoBase;
	BYTE		bIoRange;
	} IsaResFixedIo;

typedef struct
	{
	IsaSmallTag tag;
	BYTE		bChecksum;
	} IsaResEnd;

/* Large Resource Items */

typedef struct
	{
	BYTE	bLargeTagField 	: 7;	// Bits 6:0 - Large Tag Type
	BYTE	bTagType 		: 1;	// Bit 7	- Small = 0, Large = 1
	WORD	wLength;
	} IsaLargeTag;

typedef struct
	{
	IsaLargeTag tag;
	BYTE		bMemInfo;
	WORD		wMemBaseMin;		// Notes bits 7:0 are assumed to be 0.
	WORD		wMemBaseMax;		//  "                              "
	WORD		wMemAlignment;		// 0x0000 == 64 KB
	WORD		wMemRange;			// In 256 bytes increments.
	} IsaResMemory;

typedef struct
	{
	IsaLargeTag tag;
	BYTE		bData;
	} IsaResAnsiString;

typedef struct
	{
	IsaLargeTag tag;
	WORD		wCountry;
	BYTE		bData;
	} IsaResUnicodeString;

typedef struct
	{
	IsaLargeTag tag;
	BYTE		bMemInfo;
	DWORD		dwMemBaseMin;
	DWORD		dwMemBaseMax;
	DWORD		dwMemAlignment;
	DWORD		dwMemRange;
	} IsaResMem32;

typedef struct
	{
	IsaLargeTag tag;
	BYTE		bMemInfo;
	DWORD		dwMemBase;
	DWORD		dwMemRange;
	} IsaResMem32Fixed;

typedef union
	{
	IsaResPnPVersion	ver;
	IsaResLogicalId		lid;
	IsaResCompatId		cid;
	IsaResIrq			irq;
	IsaResDma			dma;
	IsaResStartDep		sdf;
	IsaResEndDep		edf;
	IsaResIo			io;
	IsaResFixedIo		fio;
	IsaResEnd			end;
	IsaResMemory		mem;
	IsaResAnsiString	ans;
	IsaResUnicodeString	uni;
	IsaResMem32			m32;
	IsaResMem32Fixed	m32f;
	} IsaRes;


 /*************************************************************************
 *                    Function Prototypes
 **************************************************************************/

#ifdef PNPISA_CPP

#else

extern void DumpIsaResources (BYTE **ppbIsa);
extern void DumpIsaSumResources (BYTE **ppbIsa, int iLongSum);

#endif

 /*************************************************************************
 *                    Variable Declarations
 **************************************************************************/



#endif
/* END OF FILE - PNPISA.H */
