 /*
 *
 * TITLE   UTILS.CPP  -  Misc Routines
 *
 * Copyright (c) 1995 by Glenn Jystad, All Rights Reserved.
 *
 ***************************************************************************
 *
 * Filename:         UTILS.CPP
 *
 **************************************************************************/

#define UTILS_CPP

 /*************************************************************************
 *                    Include Files
 **************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "STDDEFS.H"

 /*************************************************************************
 *                    Local Defines
 **************************************************************************/

 /*************************************************************************
 *                    Global Variable Declarations
 **************************************************************************/

 /*************************************************************************
 *                    Functions
 **************************************************************************/

 /***************************************************************************
 *
 * Function Name: PressKeyToContinue
 *
 * Description:
 * Parameters:
 * Returns:
 *
 * Notes:
 *
 *************************************************************************/
void PressKeyToContinue ()
	{

	printf ("Press a key to continue.");
	char ch=getchar();
	printf ("\r                       ");
	printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");

	} // end PressKeyToContinue


 /***************************************************************************
 *
 * Function Name: DumpChars
 *
 * Description:   Displays an non null terminated string
 * Parameters:    int iLen, BYTE * pbData
 * Returns:       void
 *
 *************************************************************************/
void DumpChars (int iLen, BYTE * pbData)
	{

	for (int i=0; i<iLen; i++)
		{
		printf ("%c", pbData[i]);
		}// end for

	} // end int


 /***************************************************************************
 *
 * Function Name: GetBit
 *
 * Description:   Determines lowest bit position set.
 * Parameters:    WORD wMask
 * Returns:       int  iPos
 *
 *************************************************************************/
int GetBit (WORD wMask)
	{
	int iPos = 0;

	if (wMask)					// Make sure there is a bit set.
		{
		while ( !(wMask&0x01))
			{
			iPos++;				// Increment index count.
			wMask >>= 1;		// Shift to the left one position.
			}// end while
		}// end if

	return (iPos);

	} // end GetBit


 /***************************************************************************
 *
 * Function Name: GetAndClearBit
 *
 * Description:   Determines lowest bit position set.
 *				  Return its decimal value and clear it in the source.
 * Parameters:    WORD* pwMask
 * Returns:       int  iPos
 *
 *************************************************************************/
int GetAndClearBit (WORD* pwMask)
	{
	int iPos = 0;
	WORD wMask = *pwMask;
	WORD wClearMask = 1;

	if (!wMask)					// Make sure there is a bit set.
		{
		return 0xFF;			// Indicate an Error.
		}// end else

	while ( !(wMask&0x01))  	// While bit 0 is not set...
		{
		iPos++;					// Increment index count.
		wMask >>= 1;			// Shift to the left one position.
		wClearMask <<= 1;		// Shift bit position too.
		}// end while

	*pwMask ^= wClearMask;		// Clear bit just found.

	return (iPos);

	} // end GetAndClearBit


 /***************************************************************************
 *
 * Function Name: PrintDebugOutput
 *
 * Description:   Display buffer in DEBUG like format
 * Parameters:    BYTE * pbBuf, int iLength
 * Returns:       void
 *
 * Notes:
 *
 *************************************************************************/
void PrintDebugOutput (BYTE *pbBuf, int iLength)
	{
	int i=0;
	int iLineCount = 0;

	while (i < iLength)
		{
		printf ("    %04Xh :", i);

		for (int j=0; j < 8; j++)
			{
			(i+j < iLength ? printf (" %02X", pbBuf[i+j]) :
		    				 printf ("   ") );
			}// end for

		printf ("-");

		for (j=8; j < 16; j++)
			{
			(i+j < iLength ? printf ("%02X ", pbBuf[i+j]) :
		    				 printf ("   ") );
			}// end for

		printf (" ");

		for (j=0; j < 16; j++)
			{
			if (i+j < iLength)
				{
				(pbBuf[i+j] >= ' ' ? printf ("%c", pbBuf[i+j]) :
									 printf ("."));

				}// end if
			else
				{
				printf (" ");
				}// end else

			}// end for

		i += 16;
		printf ("\n");
		iLineCount++;

		if ((iLineCount % 25) == 0)
			{
			PressKeyToContinue();
			}// end if

		}// end while

	} // end PrintDebugOutput


 /***************************************************************************
 *
 * Function Name: EisaId
 *
 * Description:   Convert a DWORD compressed ID into a 7 character string.
 * Parameters:    DWORD dwID, char * pcbuf
 * Returns:       Pointer to pcbuf
 *
 * Notes:
 *
 *************************************************************************/
char * EisaId (DWORD dwCompressedID, char * pszBuffer)
	{
	WORD	wManufacturer;
	WORD	wProduct;
	int	i;
	char	c;
	char * pcReturnBuf = pszBuffer;

	// Clear string buffer
	for (i=0; i<8; pszBuffer[i] = 0, i++);

	// Get bytes 0 and 1 (manufacturer's 3 char compressed word)
	wManufacturer = (WORD) (((dwCompressedID >> 8) & 0xFF) |
							(dwCompressedID << 8));

	// Fill string buffer with manufacturer's characters.
	for (i = 1; i <= 3; i++)
		{
		char c1 = (char)(((wManufacturer >> (15-5*i)) & 0x1F) +
						'A' - 1);
		*pszBuffer++ = c1;
		}

	// Get bytes 2 and 3 (product and version number)
	wProduct = (WORD) ((dwCompressedID >> 24) |
						((dwCompressedID >> 8) &
						0xFF00));

	// Fill string with 4 product version numbers.
	for (i = 1; i <= 4; i++)
		{
		c = (char)((wProduct >> (16-4*i)) & 0x0F);
		*pszBuffer++ = (char)(c + (c >= 0xA ? 0x37 : (char)0x30));
		}

	return (pcReturnBuf);

	} // end EisaId




/* END OF FILE - UTILS.CPP */
